<?php

class HappyForms_Admin_Stats {

	private static $instance;

	public $action_filter = 'happyforms_stats_filter';
	public $action_reset = 'happyforms_stats_reset';
	public $option = 'happyforms_stat_settings';
	public $form_id = '';
	public $span = '';
	public $show = true;
	public $show_option_name = 'happyforms-show-stats';
	public $starts_from = '1970-01-01 00:00:00';
	public $widgets = array();

	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		self::$instance->hook();

		return self::$instance;
	}

	public function hook() {
		add_action( 'load-edit.php', array( $this, 'init' ) );
		add_action( 'wp_ajax_' . $this->action_filter, array( $this, 'filter' ) );
		add_action( 'wp_ajax_' . $this->action_reset, array( $this, 'reset' ) );
		add_action( 'happyforms_form_deleted', array( $this, 'form_deleted' ) );
	}

	public function is_enabled() {
		$enabled = apply_filters( 'happyforms_logger_enabled', true );

		return $enabled;
	}

	public function init() {
		if ( ! $this->is_enabled() ) {
			return;
		}

		$screen = get_current_screen();

		if ( happyforms_get_message_controller()->post_type !== $screen->post_type ) {
			return;
		}

		$this->configure();

		if ( $this->show ) {
			add_action( 'admin_notices', array( $this, 'render' ) );
		}

		add_action ( 'admin_body_class', array( $this, 'body_class' ) );
		add_filter( 'screen_settings', array( $this, 'render_screen_settings' ), 20, 2 );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
	}

	public function configure() {
		$this->get_settings();
		$this->register_widgets();
		$this->update_settings();
	}

	public function get_defaults() {
		$defaults = array(
			'show' => true,
			'starts_from' => '1970-01-01 00:00:00',
		);

		return $defaults;
	}

	public function get_settings() {
		$settings = get_option( $this->option, $this->get_defaults() );

		$this->show = $settings['show'];

		if ( isset( $_REQUEST[ 'screenoptionnonce' ] ) ) {
			$this->show = isset( $_REQUEST[ $this->show_option_name ] );
		}

		$this->starts_from = $settings['starts_from'];
		$this->form_id = $this->get_form_filter();
		$this->span = $this->get_span_filter();
	}

	public function form_deleted( $form_id ) {
		if ( ! $this->is_enabled() ) {
			return;
		}
		
		global $wpdb;

		$options = $wpdb->get_results( "SELECT option_name FROM $wpdb->options", ARRAY_A );
		$options = wp_list_pluck( $options, 'option_name' );

		foreach( $options as $option ) {
			if ( preg_match( '/happyforms_stat_[^\d]+' . $form_id . '/', $option ) ) {
				delete_option( $option );
			}
		}
	}

	public function update_settings() {
		update_option( $this->option, array(
			'show' => $this->show,
			'starts_from' => $this->starts_from,
		) );
	}

	public function get_form_filter() {
		$forms = happyforms_get_form_controller()->get();
		$form_ids = wp_list_pluck( $forms, 'ID' );
		$form_id = '';

		if ( isset( $_POST['form_id'] ) && in_array( $_POST['form_id'], $form_ids ) ) {
			$form_id = $_POST['form_id'];
		}

		return $form_id;
	}

	public function get_span_filter() {
		$spans = array( 'daily', 'weekly', 'monthly' );
		$span = 'weekly';

		if ( isset( $_POST['span'] ) && in_array( $_POST['span'], $spans ) ) {
			$span = $_POST['span'];
		}

		return $span;
	}

	public function register_widget( $title, $class ) {
		$instance = new $class( $this->starts_from, $this->form_id );

		switch( $this->span ) {
			case 'daily':
				$values = $instance->daily;
				break;
			case 'monthly':
				$values = $instance->monthly;
				break;
			default:
				$values = $instance->weekly;
				break;
		}

		$trend = $instance->trend( $values );
		$highest = call_user_func_array( 'max', $values );
		$bars = array_map( function( $value ) use( $highest ) {
			$bar = $highest > 0 ? ceil( $value / $highest * 100 ) : 1;
			$bar = 1 >= $bar ? "2px" : "$bar%";
			return $bar;
		}, $values );

		$this->widgets[] = array(
			'title' => $title,
			'id' => $instance->name,
			'values' => $values,
			'bars' => $bars,
			'trend' => $trend,
		);
	}

	public function register_widgets() {
		$this->register_widget(
			__( 'All submitted', 'happyforms_upgrade' ),
			'HappyForms_Stat_ResponsesStarted'
		);

		$this->register_widget(
			__( 'All completed', 'happyforms_upgrade' ),
			'HappyForms_Stat_ResponsesSubmitted'
		);

		$this->register_widget(
			__( 'All abandoned', 'happyforms_upgrade' ),
			'HappyForms_Stat_ResponsesAbandoned'
		);

		$this->register_widget(
			__( 'From mobile', 'happyforms_upgrade' ),
			'HappyForms_Stat_ResponsesMobile'
		);

		$this->register_widget(
			__( 'Validation errors', 'happyforms_upgrade' ),
			'HappyForms_Stat_ValidationErrors'
		);

		$this->register_widget(
			__( 'New contacts', 'happyforms_upgrade' ),
			'HappyForms_Stat_NewContacts'
		);
	}

	public function render() {
		$action_filter = $this->action_filter;
		$action_reset = $this->action_reset;
		$widgets = $this->widgets;

		require_once( happyforms_get_include_folder() . '/templates/partials/admin-stats-wrapper.php' );
	}

	public function filter() {
		if ( ! $this->is_enabled() ) {
			return;
		}

		if ( ! check_ajax_referer( $this->action_filter ) ) {
			wp_die( _e( 'An error occurred.', 'happyforms' ) );
		}

		$this->configure();
		$this->output_widgets();
	}

	public function reset() {
		if ( ! $this->is_enabled() ) {
			return;
		}

		if ( ! check_ajax_referer( $this->action_reset ) ) {
			wp_die( _e( 'An error occurred.', 'happyforms' ) );
		}

		$this->starts_from = ( new DateTime( 'today' ) )->format( 'Y-m-d H:i:s' );

		$this->update_settings();
		$this->configure();
		$this->output_widgets();
	}

	public function output_widgets() {
		$action_filter = $this->action_filter;
		$action_reset = $this->action_reset;
		$widgets = $this->widgets;

		require_once( happyforms_get_include_folder() . '/templates/partials/admin-stats.php' );
		exit();
	}

	public function body_class( $classes ) {
		if ( $this->show ) {
			$classes .= ' happyforms-has-stats';
		}

		return $classes;
	}

	public function render_screen_settings( $settings, $screen ) {
		ob_start();
		?>
		<fieldset style="display: block;" class="happyforms-analytics-settings">
			<legend><?php _e( 'Analytics', 'happyforms' ); ?></legend>
			<label>
				<input class="hide-column-tog" name="<?php echo $this->show_option_name; ?>" type="checkbox" value="1" <?php checked( $this->show ); ?>><?php _e( 'Show analytics', 'happyforms' ); ?>
			</label>
		</fieldset>
		<?php
		$content = ob_get_clean();

		return $settings . $content;
	}

	public function enqueue_scripts() {
		wp_enqueue_script(
			'happyforms-frontend',
			happyforms_get_plugin_url() . 'inc/assets/js/admin/stats.js',
			array( 'jquery' ), HAPPYFORMS_UPGRADE_VERSION, true
		);
	}

}

if ( ! function_exists( 'happyforms_get_admin_stats' ) ):

function happyforms_get_admin_stats() {
	return HappyForms_Admin_Stats::instance();
}

endif;

happyforms_get_admin_stats();
