<?php
class HappyForms_Form_Password_Protection {
	private static $instance;

	private $password_meta_key = 'password';

	private $frontend_styles = false;

	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		self::$instance->hook();

		return self::$instance;
	}

	public function hook() {
		add_filter( 'happyforms_meta_fields', array( $this, 'add_fields' ), 10, 1 );
		add_filter( 'happyforms_setup_controls', array( $this, 'add_setup_controls' ), 10, 1 );
		add_action( 'happyforms_do_setup_control', array( $this, 'do_control' ), 10, 3 );
		add_filter( 'happyforms_form_template_path', array( $this, 'set_password_template_path'), 10, 2 );
		add_filter( 'happyforms_get_steps', array( $this, 'steps_add_password' ), 10, 2 );
		add_action( 'happyforms_step', array( $this, 'add_password_step' ) );
		add_filter( 'happyforms_get_form_data', array( $this, 'get_password_field' ), 10, 1 );
		add_filter( 'happyforms_update_form_data', array( $this, 'hash_password_field' ), 10, 1 );
		add_action( 'happyforms_customize_enqueue_scripts', array( $this, 'customize_enqueue_scripts' ) );
		add_action( 'happyforms_print_frontend_styles', array( $this, 'print_frontend_styles' ), 10, 1 );
		add_filter( 'happyforms_session_reset_callback', array( $this, 'set_session_reset_callback' ), 10, 3 );
	}

	public function add_fields( $fields ) {
		$fields['password_protect'] = array(
			'default'  => 0,
			'sanitize' => 'happyforms_sanitize_checkbox'
		);

		$fields[$this->password_meta_key] = array(
			'default'  => '',
			'sanitize' => ''
		);

		$fields['password_input_placeholder'] = array(
			'default'  => __( 'Enter your password', 'happyforms' ),
			'sanitize' => 'sanitize_text_field'
		);

		$fields['password_submit_button_label'] = array(
			'default'  => __( 'Access Form', 'happyforms' ),
			'sanitize' => 'sanitize_text_field'
		);

		$fields['password_error_message'] = array(
			'default'  => __( 'Oops! The password you entered is incorrect. Please try again.', 'happyforms' ),
			'sanitize' => 'esc_html',
		);

		return $fields;
	}

	public function add_setup_controls( $controls ) {
		$controls[1550] = array(
			'field'   => 'password_protect',
			'label'   => __( 'Require password', 'happyforms' ),
			'tooltip' => __( 'Only users with password will be able to view and submit the form.', 'happyforms' ),
			'type'    => 'checkbox'
		);

		$controls[1551] = array(
			'type' => 'group_start',
			'trigger' => 'password_protect'
		);

		$controls[1552] = array(
			'field'     => '',
			'type'      => 'section_start',
			'id'        => 'happyforms-password-protect-settings',
			'show_when' => 'password_protect',
		);

		$controls[1553] = array(
			'field'        => $this->password_meta_key,
			'label'        => __( 'Password', 'happyforms' ),
			'label_filled' => __( 'New password', 'happyforms' ),
			'type'         => 'password'
		);

		$controls[1554] = array(
			'field' => 'password_input_placeholder',
			'label' => __( 'Input placeholder', 'happyforms' ),
			'type'  => 'text',
			'autocomplete' => 'off',
		);

		$controls[1555] = array(
			'field' => 'password_submit_button_label',
			'label' => __( 'Submit password button label', 'happyforms' ),
			'type'  => 'text',
			'autocomplete' => 'off',
		);

		$controls[1556] = array(
			'field' => 'password_error_message',
			'label' => __( 'Error message', 'happyforms' ),
			'type' => 'editor',
		);

		$controls[1557] = array(
			'type'  => 'group_end'
		);

		return $controls;
	}

	public function set_password_template_path( $template_path, $form ) {
		if ( $this->requires_password( $form ) && 'password' === happyforms_get_current_step( $form ) ) {
			$template_path = happyforms_get_include_folder() . '/templates/single-form-password.php';
		}

		return $template_path;
	}

	public function requires_password( $form ) {
		return ( 1 === intval( happyforms_get_form_property( $form, 'password_protect' ) ) );
	}

	public function steps_add_password( $steps, $form ) {
		if ( $this->requires_password( $form ) ) {
			$steps[0] = 'password';
		}

		return $steps;
	}

	public function customize_enqueue_scripts() {
		wp_enqueue_script(
			'happyforms-password-protect',
			happyforms_get_plugin_url() . 'inc/assets/js/customize/password-protect.js',
			array( 'happyforms-customize' ),
			HAPPYFORMS_UPGRADE_VERSION,
			true
		);
	}

	public function add_password_step( $form ) {
		if ( 'password' !== happyforms_get_current_step( $form ) ) {
			return;
		}

		$session = happyforms_get_session();
		$form_id = $form['ID'];
		$form_controller = happyforms_get_form_controller();
		$message_controller = happyforms_get_message_controller();
		$submission = $this->validate_form_password( $form, $_REQUEST );

		$response = array();

		if ( false === $submission ) {
			$error_message = $form['password_error_message'];
			$error_message = html_entity_decode( $error_message );
			$error_message = wp_unslash( $error_message );

			$session->add_error( $form_id, $error_message );
			$session->reset_step();

			$response['html'] = $form_controller->render( $form );

			// Send error response
			wp_send_json_error( $response );
		} else {
			$session->next_step();

			// Render the form
			$response['html'] = $form_controller->render( $form );

			$message_controller->send_json_success( $response, $submission, $form );
		}

		// Send success response
		$message_controller->send_json_success( $response, array(), $form );
	}

	public function validate_form_password( $form, $request = array() ) {
		if ( ! isset( $request['happyforms_password'] ) || empty( $request['happyforms_password'] ) ) {
			return false;
		}

		if ( wp_check_password( $request['happyforms_password'], happyforms_get_meta( $form['ID'], 'password', true ) ) ) {
			return true;
		}

		return false;
	}

	public function get_password_field( $form_array ) {
		if ( isset( $form_array[$this->password_meta_key] ) ) {
			$form_array[$this->password_meta_key] = '';
		}

		return $form_array;
	}

	public function hash_password_field( $update_data ) {
		$prefixed_meta = '_happyforms_' . $this->password_meta_key;

		if ( isset( $update_data[$prefixed_meta] ) ) {
			$value = $update_data[$prefixed_meta];

			if ( ! empty( $value ) ) {
				$hashed_password = wp_hash_password( $value );
				$update_data[$prefixed_meta] = $hashed_password;
			} else {
				unset( $update_data[$prefixed_meta] );
			}
		}

		return $update_data;
	}

	public function do_control( $control, $field, $index ) {
		$type = $control['type'];

		if ( 'password' === $type ) {
			require( happyforms_get_include_folder() . '/templates/customize-controls/password.php' );
		}
	}

	public function print_frontend_styles( $form ) {
		$output = apply_filters( 'happyforms_enqueue_style', true );

		if ( $output && $this->requires_password( $form ) && ! $this->frontend_styles ) {
			$this->frontend_styles = true;
			$url = happyforms_get_plugin_url() . 'inc/assets/css/frontend/password.css?' . HAPPYFORMS_UPGRADE_VERSION;
			?>
			<link rel="stylesheet" property="stylesheet" href="<?php echo $url; ?>" />
			<?php
		}
	}

	public function set_session_reset_callback( $params, $session, $form ) {
		if ( ! $this->requires_password( $form ) ) {
			return $params;
		}

		$params['function'] = array( $session, 'set_step' );
		$params['args']     = array( 1 );

		return $params;
	}


}

if ( ! function_exists( 'happyforms_upgrade_get_password_protection' ) ) :

	function happyforms_upgrade_get_password_protection() {
		return HappyForms_Form_Password_Protection::instance();
	}

endif;

happyforms_upgrade_get_password_protection();
