<?php
class HappyForms_Settings_Controller {

	private static $instance;

	private $settings;
	private $update_authorize_action =  'happyforms-updates-authorize';
	private $update_deauthorize_action =  'happyforms-updates-deauthorize';
	private $manage_sections_states_action = 'happyforms-settings-manage-sections-states';
	private $section_states_meta_key = 'happyforms-settings-sections-states';

	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		self::$instance->hook();

		return self::$instance;
	}

	public function hook() {
		add_action( 'init', array( $this, 'set_settings' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
		add_action( 'wp_ajax_' . $this->update_authorize_action, array( $this, 'updater_authorize' ) );
		add_action( 'wp_ajax_' . $this->update_deauthorize_action, array( $this, 'updater_deauthorize' ) );
		add_action( 'wp_ajax_' . $this->manage_sections_states_action, array( $this, 'manage_sections_states' ) );

		add_filter( 'happyforms_settings_page_method', array( $this, 'set_admin_page_method' ) );
		add_filter( 'happyforms_settings_page_url', array( $this, 'set_admin_page_url' ) );
		add_filter( 'ttf_product_central_settings_template', array( $this, 'set_updater_template' ) );
		add_filter( 'happyforms_settings_page_menu_badge', array( $this, 'handle_settings_page_menu_badge' ) );
	}

	public function set_settings() {
		$default_sections = array(
			10 => array(
				'id' => 'validation-messages',
				'title' => __( 'Validation Messages', 'happyforms' ),
				'description' => __( 'Revise the wording or language of messages displayed to users when submitting a form with errors.', 'happyforms' ),
				'template' => happyforms_get_include_folder() . '/templates/admin-settings-validation-messages.php',
				'default_state' => 'open',
				'column' => 1,
			),
			20 => array(
				'id' => 'export',
				'title' => __( 'Import and Export', 'happyforms' ),
				'description' => __( 'Transfer forms and submissions to and from sites.', 'happyforms' ),
				'template' => happyforms_get_include_folder() . '/templates/admin-settings-export.php',
				'default_state' => 'open',
				'column' => 1,
			),
			30 => array(
				'id' => 'register',
				'title' => __( 'Register', 'happyforms' ),
				'template' => happyforms_get_include_folder() . '/templates/admin-settings-register.php',
				'default_state' => 'open',
				'column' => 1,
			),
			// 40 => array(
			// 	'id' => 'blocklist',
			// 	'title' => __( 'Submission Blocklist', 'happyforms' ),
			// 	'description' => __( 'Add spammy words, numbers, email addresses, and IP addresses below (one per line, please). New submissions will then be scanned against this list and if an exact match is found that submission will automatically be marked as spam.', 'happyforms' ),
			// 	'template' => happyforms_get_include_folder() . '/templates/admin-settings-blocklist.php',
			// 	'default_state' => 'open',
			// 	'column' => 2,
			// )
		);

		$sections = apply_filters( 'happyforms_get_settings_sections', $default_sections );
		ksort( $sections );

		$this->settings = $sections;
	}

	public function get_sections( $column = null ) {
		if ( is_null( $column ) ) {
			return $this->settings;
		}

		$column_settings = array_filter( $this->settings, function( $section ) use ( $column ) {
			return $column === $section['column'];
		} );

		return $column_settings;
	}

	public function set_admin_page_method() {
		return array( $this, 'settings_page' );
	}

	public function set_admin_page_url() {
		return 'happyforms-settings';
	}

	public function settings_page() {
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die( __( 'Sorry, you are not allowed to access this page.', 'happyforms' ) );
		}

		add_filter( 'admin_footer_text', 'happyforms_admin_footer' );

		require_once( happyforms_get_include_folder() . '/templates/admin-settings.php' );
	}

	public function handle_settings_page_menu_badge() {
		$badge = '';

		$products = ttf_product_central_updater()->get_products();
		$data = happyforms_get_plugin_metadata();
		$product_id = $data['id'];
		$product = $products[$product_id];

		if ( ! $product->is_authorized() ) {
			$badge = ' <span class="awaiting-mod count-1"><span class="pending-count">1</span></span>';
		}

		return $badge;
	}

	public function set_updater_template( $template ) {
		$template = happyforms_get_include_folder() . '/templates/admin-settings-register-form.php';

		return $template;
	}

	public function updater_authorize() {
		ttf_product_central_settings()->authorize( 'ttf-updates-happyforms' );
		ttf_product_central_settings()->render_page( 'ttf-updates-happyforms' );
		exit;
	}

	public function updater_deauthorize() {
		ttf_product_central_settings()->deauthorize( 'ttf-updates-happyforms' );
		ttf_product_central_settings()->render_page( 'ttf-updates-happyforms' );
		exit;
	}

	public function admin_enqueue_scripts() {
		wp_enqueue_script(
			'happyforms-admin-settings-screen',
			happyforms_get_plugin_url() . 'inc/assets/js/admin/settings.js',
			array(), HAPPYFORMS_UPGRADE_VERSION, true
		);

		wp_localize_script(
			'happyforms-admin-settings-screen',
			'_happyFormsSettingsPageConfig',
			array(
				'manage_sections_state_nonce' => wp_create_nonce( $this->manage_sections_states_action )
			)
		);
	}

	public function manage_sections_states() {
		if ( ! isset( $_REQUEST['nonce'] ) ) {
			wp_send_json_error();
		}

		if ( ! wp_verify_nonce( $_REQUEST['nonce'], $this->manage_sections_states_action ) ) {
			wp_send_json_error();
		}

		$section_id = sanitize_text_field( $_REQUEST['section_id'] );
		$section_state = sanitize_text_field( $_REQUEST['section_state'] );
		$current_user_id = get_current_user_id();
		$user_sections_states = $this->get_user_sections_states( $current_user_id );
		$section_key = array_search( $section_id, $user_sections_states );

		$user_sections_states[$section_id] = $section_state;

		update_user_meta(
			$current_user_id,
			$this->section_states_meta_key,
			$user_sections_states
		);

		wp_send_json_success();
	}

	public function get_user_sections_states( $user_id = 0 ) {
		if ( 0 === $user_id ) {
			$user_id = get_current_user_id();
		}

		$sections_states = get_user_meta(
			$user_id,
			$this->section_states_meta_key,
			true
		);

		if ( empty( $sections_states ) ) {
			$sections_states = array();
		}

		return $sections_states;
	}

}

if ( ! function_exists( 'happyforms_get_settings_controller' ) ):

function happyforms_get_settings_controller() {
	return HappyForms_Settings_Controller::instance();
}

endif;

happyforms_get_settings_controller();
