<?php

class HappyForms_Stat {

	protected $option;

	public $name;
	public $starts_from;
	public $last_updated;
	public $form_id;
	public $daily;
	public $weekly;
	public $monthly;

	public function __construct( $starts_from, $form_id = '' ) {
		$this->option = 'happyforms_stat_' . $this->name;
		$this->starts_from = $starts_from;
		$this->form_id = $form_id;

		if ( ! empty( $this->form_id ) ) {
			$this->option .= '_' . $form_id;
		}

		$this->read();
		$this->check_updates();
	}

	public function get_defaults() {
		$defaults = array(
			'starts_from' => '1970-01-01 00:00:00',
			'last_updated' => '1970-01-01 00:00:00',
			'daily' => array_fill( 0, 7, 0 ),
			'weekly' => array_fill( 0, 4, 0 ),
			'monthly' => array_fill( 0, 12, 0 ),
		);

		return $defaults;
	}

	public function read() {
		$views = get_option( $this->option, $this->get_defaults() );
		$needs_reset = ( $this->starts_from !== $views['starts_from'] );

		$this->last_updated = $views['last_updated'];
		$this->daily = $views['daily'];
		$this->weekly = $views['weekly'];
		$this->monthly = $views['monthly'];

		if ( $needs_reset ) {
			$this->reset();
		}
	}

	public function write() {
		$views = array(
			'starts_from' => $this->starts_from,
			'last_updated' => $this->last_updated,
			'daily' => $this->daily,
			'weekly' => $this->weekly,
			'monthly' => $this->monthly,
		);

		update_option( $this->option, $views, false );
	}

	public function update() {
		$this->last_updated = ( new DateTime( 'today' ) )->format( 'Y-m-d H:i:s' );

		$this->calculate_daily();
		$this->calculate_weekly();
		$this->calculate_monthly();
		$this->write();
	}

	public function reset() {
		$this->last_updated = $this->starts_from;
		$this->daily = array_fill( 0, 7, 0 );
		$this->weekly = array_fill( 0, 4, 0 );
		$this->monthly = array_fill( 0, 12, 0 );

		$this->write();
	}

	public function get_defaults_daily() {
		$time = new DateTime( 'now' );
		$intervals = array();

		foreach( range( 1, 7 ) as $diff ) {
			$key = $time->modify( '-1 day' )->format( 'Y-m-d' );
			$intervals[$key] = 0;
		}

		return $intervals;
	}

	public function get_defaults_weekly() {
		$time = new DateTime( 'now' );
		$intervals = array();

		foreach( range( 1, 28 ) as $diff ) {
			$key = $time->modify( '-1 day' )->format( 'Y-m-d' );
			$intervals[$key] = 0;
		}

		return $intervals;
	}

	public function get_defaults_monthly() {
		$time = new DateTime( 'now' );
		$intervals = array();

		foreach( range( 1, 360 ) as $diff ) {
			$key = $time->modify( '-1 day' )->format( 'Y-m-d' );
			$intervals[$key] = 0;
		}

		return $intervals;
	}

	public function get_query_join() {
		if ( empty( $this->form_id ) ) {
			return '';
		}

		global $wpdb;

		$join = "
			JOIN $wpdb->postmeta m
			ON p.ID = m.post_id
			AND m.meta_key = '_happyforms_form_id'
		";

		return $join;
	}

	public function get_query_where() {
		if ( empty( $this->form_id ) ) {
			return '';
		}

		$where = "
			AND m.meta_value = '$this->form_id'
		";

		return $where;
	}

	public function calculate( $format, $from, $to ) {
		global $wpdb;

		$starts_from = $this->starts_from;
		$join = $this->get_query_join();
		$where = $this->get_query_where();

		$query = "
			SELECT DATE_FORMAT(p.post_date, '$format') AS date, COUNT(p.id) AS count
			FROM $wpdb->posts p
			$join
			WHERE p.post_type = 'happyforms-message'
			AND p.post_date >= '$starts_from'
			AND p.post_date >= '$from'
			AND p.post_date < '$to'
			$where
			GROUP BY date;
		";

		$results = $wpdb->get_results( $query, ARRAY_A );
		$results = wp_list_pluck( $results, 'count', 'date' );

		return $results;
	}

	public function calculate_daily() {
		global $wpdb;

		$defaults = $this->get_defaults_daily();
		$from = ( new DateTime( 'now' ) )->modify( '-7 day' )->format( 'Y-m-d' );
		$to = ( new DateTime( 'now' ) )->format( 'Y-m-d' );
		$results = $this->calculate( '%Y-%m-%d', $from, $to );
		$results = wp_parse_args( $results, $defaults );
		$results = array_values( $results );

		$this->daily = $results;
	}

	public function calculate_weekly() {
		global $wpdb;

		$defaults = $this->get_defaults_weekly();
		$from = ( new DateTime( 'now' ) )->modify( '-28 day' )->format( 'Y-m-d' );
		$to = ( new DateTime( 'now' ) )->format( 'Y-m-d' );
		$results = $this->calculate( '%Y-%m-%d', $from, $to );
		$results = wp_parse_args( $results, $defaults );
		$results = array_values( $results );
		$results = array_chunk( $results, 7 );
		$results = array_map( 'array_sum', $results );

		$this->weekly = $results;
	}

	public function calculate_monthly() {
		global $wpdb;

		$defaults = $this->get_defaults_monthly();
		$from = ( new DateTime( 'now' ) )->modify( '-360 day' )->format( 'Y-m-d' );
		$to = ( new DateTime( 'now' ) )->format( 'Y-m-d' );
		$results = $this->calculate( '%Y-%m-%d', $from, $to );
		$results = wp_parse_args( $results, $defaults );
		$results = array_values( $results );
		$results = array_chunk( $results, 30 );
		$results = array_map( 'array_sum', $results );

		$this->monthly = $results;
	}

	public function trend( $set ) {
		$current = $set[0];
		$previous = $set[1];
		$difference = $current - $previous;
		$sign = ( $difference >= 0 ) ? '+' : '-';
		$previous = 0 !== $previous ? $previous : 1;
		$trend = round( $difference / $previous * 100 );

		return $trend;
	}

	public function check_updates() {
		$today = new DateTime( 'today' );
		$last_updated = new DateTime( $this->last_updated );
		$difference = $today->diff( $last_updated )->days;

		if ( $difference > 0 ) {
			$this->update();
		}
	}

}

class HappyForms_Stat_ResponsesStarted extends HappyForms_Stat {

	public $name = 'responses_started';

}

class HappyForms_Stat_ResponsesSubmitted extends HappyForms_Stat {

	public $name = 'responses_submitted';

	public function get_query_where() {
		$where = "
			AND p.post_status = 'publish'
		";

		if ( ! empty( $this->form_id ) ) {
			$where .= "
				AND m.meta_value = '$this->form_id'
			";
		}

		return $where;
	}

}

class HappyForms_Stat_ResponsesAbandoned extends HappyForms_Stat {

	public $name = 'responses_abandoned';

	public function get_query_where() {
		$where = "
			AND p.post_status = 'draft'
		";

		if ( ! empty( $this->form_id ) ) {
			$where .= "
				AND m.meta_value = '$this->form_id'
			";
		}

		return $where;
	}

}

class HappyForms_Stat_ResponsesMobile extends HappyForms_Stat {

	public $name = 'responses_mobile';

	public function get_query_join() {
		global $wpdb;

		$join = "
			JOIN $wpdb->postmeta mm
			ON p.ID = mm.post_id
			AND mm.meta_key = '_happyforms_client_platform'
			AND mm.meta_value = 'mobile'
		";

		if ( ! empty( $this->form_id ) ) {
			$join .= "
				JOIN $wpdb->postmeta m
				ON p.ID = m.post_id
				AND m.meta_key = '_happyforms_form_id'
			";
		}

		return $join;
	}

}

class HappyForms_Stat_ValidationErrors extends HappyForms_Stat {

	public $name = 'validation_errors';

	public function get_query_where() {
		$where = "
			AND comment_content = 'error'
		";

		if ( ! empty( $this->form_id ) ) {
			$where .= "
				AND comment_post_ID = '$this->form_id'
			";
		}

		return $where;
	}

	public function calculate( $format, $from, $to ) {
		global $wpdb;

		$starts_from = $this->starts_from;
		$where = $this->get_query_where();

		$query = "
			SELECT DATE_FORMAT(comment_date, '$format') AS date, COUNT(comment_ID) AS count
			FROM $wpdb->comments
			WHERE comment_type = 'happyforms-event'
			AND comment_date >= '$starts_from'
			AND comment_date >= '$from'
			AND comment_date < '$to'
			$where
			GROUP BY date;
		";

		$results = $wpdb->get_results( $query, ARRAY_A );
		$results = wp_list_pluck( $results, 'count', 'date' );

		return $results;
	}

}

class HappyForms_Stat_NewContacts extends HappyForms_Stat_ValidationErrors {

	public $name = 'new_contacts';

	public function get_query_where() {
		$where = "
			AND comment_content = 'lead'
		";

		if ( ! empty( $this->form_id ) ) {
			$where .= "
				AND comment_post_ID = '$this->form_id'
			";
		}

		return $where;
	}

}
