<?php

class HappyForms_Service_Mailchimp extends HappyForms_Service {

	public $id = 'mailchimp';
	public $group = 'email';

	private $api_url = '';
	private $api_call_headers;

	public $lists = array();

	public function __construct() {
		$this->label = __( 'Mailchimp', 'happyforms' );
	}

	public function get_default_credentials() {
		return array(
			'key' => '',
		);
	}

	public function set_credentials( $credentials = array(), $raw = array() ) {
		$this->credentials = wp_parse_args( $credentials, $this->get_default_credentials() );
	}

	public function is_connected() {
		$authenticated = ! empty( $this->credentials['key'] );

		return $authenticated;
	}

	public function admin_widget( $previous_credentials = array() ) {
		require_once( happyforms_get_integrations_folder() . '/services/mailchimp/partial-widget.php' );
	}

	public function configure() {
		$this->set_api_url();
	}

	public function load() {
		if ( $this->is_connected() ) {
			require_once( happyforms_get_integrations_folder() . '/services/mailchimp/class-integration-mailchimp.php' );
		}
	}

	public function set_api_url() {
		$credentials = $this->get_credentials();
		$key = $credentials['key'];

		$data_center = substr( $key, strpos( $key, '-' ) + 1, strlen( $key ) );

		$this->api_url = "https://{$data_center}.api.mailchimp.com/3.0";
		$this->api_call_headers = array(
			'Authorization' => 'Basic ' . base64_encode( "user:{$key}" )
		);
	}

	public function add_subscriber( $list_id, $subscriber_data, $tags ) {
		if ( empty( $list_id ) ) {
			return;
		}

		$merge_fields = $this->get_merge_fields( $list_id );
		$to_sanitize = array();

		foreach ( $merge_fields as $field ) {
			if ( isset( $subscriber_data['merge_fields'][$field['id']] ) && ! empty( $field['sanitize'] ) ) {
				$sanitized_value = call_user_func( $field['sanitize'], $subscriber_data['merge_fields'][$field['id']], $field['format'] );
				$subscriber_data['merge_fields'][$field['id']] = $sanitized_value;
			}
		}

		$member_request = wp_remote_post(
			"{$this->api_url}/lists/{$list_id}/members",
			array(
				'headers' => $this->api_call_headers,
				'body' => json_encode( $subscriber_data )
			)
		);

		if ( ! empty( $tags ) ) {
			$subscriber_hash = md5( strtolower( $subscriber_data['email_address'] ) );

			$tags_request = wp_remote_post(
				"{$this->api_url}/lists/{$list_id}/members/{$subscriber_hash}/tags",
				array(
					'headers' => $this->api_call_headers,
					'body' => json_encode(
						array(
							'tags' => $tags
						)
					)
				)
			);
		}
	}

	public function sanitize_date_field( $value, $format ) {
		if ( empty( $value ) || empty( $format ) ) {
			return $value;
		}

		$replace = array( 'DD', 'MM', 'YYYY' );
		$replace_with = array( 'd', 'm', 'Y' );
		$format = str_replace( $replace, $replace_with, $format );

		$date = strtotime( $value );
		$value = date( $format, $date );

		return $value;
	}

	public function get_merge_fields( $list_id ) {
		if ( empty( $list_id ) ) {
			return;
		}

		$endpoint = "{$this->api_url}/lists/{$list_id}/merge-fields";
		$response = wp_remote_get(
			$endpoint,
			array(
				'headers' => $this->api_call_headers
			)
		);

		$body = json_decode( wp_remote_retrieve_body( $response ) );
		$fields = array();

		if ( 200 !== wp_remote_retrieve_response_code( $response ) ) {
			return $fields;
		}

		if ( ! isset( $body->merge_fields ) ) {
			return $fields;
		}

		$fields[0] = array(
			'id' => 'EMAIL',
			'name' => __( 'Email Address', 'happyforms' )
		);

		foreach ( $body->merge_fields as $field ) {
			$field_array = array(
				'id' => $field->tag,
				'name' => $field->name
			);

			if ( ! empty( $field->options ) ) {
				if ( isset( $field->options->date_format ) ) {
					$field_array['sanitize'] = array( $this, 'sanitize_date_field' );
					$field_array['format'] = $field->options->date_format;
				}
			}

			if ( 'address' === $field->type ) {
				$field_array['items'] = array(
					array(
						'id' => "{$field->tag}::addr1",
						'name' => __( 'Address Line', 'happyforms' )
					),
					array(
						'id' => "{$field->tag}::city",
						'name' => __( 'City', 'happyforms' )
					),
					array(
						'id' => "{$field->tag}::state",
						'name' => __( 'State', 'happyforms' )
					),
					array(
						'id' => "{$field->tag}::zip",
						'name' => __( 'ZIP', 'happyforms' )
					),
				);
			}

			$fields[$field->display_order] = $field_array;
		}

		ksort( $fields, SORT_NUMERIC );

		return $fields;
	}

	public function get_lists() {
		if ( ! empty( $this->lists ) ) {
			return $this->lists;
		}

		$endpoint = $this->api_url . '/lists';
		$response = wp_remote_get(
			$endpoint,
			array(
				'headers' => $this->api_call_headers
			)
		);

		$body = json_decode( wp_remote_retrieve_body( $response ) );

		if ( 200 !== wp_remote_retrieve_response_code( $response ) ) {
			return $this->lists;
		}

		if ( ! isset( $body->lists ) ) {
			return $this->lists;
		}

		if ( is_array( $body->lists ) ) {
			$this->lists = $body->lists;
		}

		return $this->lists;
	}

	public function get_groups( $list_id ) {
		if ( empty( $list_id ) ) {
			return;
		}

		$endpoint = "{$this->api_url}/lists/{$list_id}/interest-categories";
		$response = wp_remote_get(
			$endpoint,
			array(
				'headers' => $this->api_call_headers
			)
		);

		$body = json_decode( wp_remote_retrieve_body( $response ) );

		$groups = array();

		if ( 200 !== wp_remote_retrieve_response_code( $response ) ) {
			return $groups;
		}

		if ( ! isset( $body->categories ) ) {
			return $groups;
		}

		foreach ( $body->categories as $group ) {
			$group_array = array(
				'id' => $group->id,
				'title' => $group->title,
				'type' => $group->type,
				'options' => array()
			);

			$interests_endpoint = "{$endpoint}/{$group->id}/interests";
			$interests_response = wp_remote_get(
				$interests_endpoint,
				array(
					'headers' => $this->api_call_headers
				)
			);

			$interests_body = $interests_response['body'];
			$interests_json = json_decode( $interests_body );

			foreach( $interests_json->interests as $interest_group ) {
				$group_array['options'][] = array(
					'value' => $interest_group->id,
					'label' => $interest_group->name
				);
			}

			$groups[] = $group_array;
		}

		return $groups;
	}

}
