( function( $, Cookies, settings ) {
	$( document ).ready( function() {
		var HappyFormsStripe = function( el ) {
			this.el = el;
			this.$el = $( el );
			this.$form = $( 'form', this.$el );
			this.$paymentsPart = $( '[data-happyforms-type=payments]', this.$form );

			this.formID = $( '[name=happyforms_form_id]', this.$form ).val();
			this.stripe = '';
			this.elements = {};
			this.checkoutDataKey = 'happyforms_' + this.formID + '_stripe_checkout';

			this.init();
		}

		HappyFormsStripe.prototype.init = function() {
			if ( ! settings.key || ! this.$paymentsPart.length ) {
				return;
			}

			if ( this.$el.attr( 'data-stripe-enabled' ) ) {
				return;
			}

			this.stripe = Stripe( settings.key, { locale: settings.locale } );
			this.$el.attr( 'data-stripe-enabled', '' );

			this.createCheckoutData();
			this.initStripeElements();

			this.$form.on( 'happyforms.submitted', this.handlePayment.bind( this ) );
			this.$paymentsPart.on( 'happyforms.cssvar', this.updatePlaceholderColor.bind( this ) );
		},

		HappyFormsStripe.prototype.updatePlaceholderColor = function( event, data ) {
			if ( ! data ) {
				return;
			}

			if ( '--happyforms-color-part-placeholder' === data.name ) {
				var args = {
					'placeholder': data.value
				};

				this.initStripeElements( args );
			}
		},

		HappyFormsStripe.prototype.initStripeElements = function( options ) {
			var elements = this.stripe.elements();

			var formComputedStyle = window.getComputedStyle( this.el );
			var partComputedStyle = window.getComputedStyle( this.$paymentsPart[0] );

			var placeholderColor = formComputedStyle.getPropertyValue( '--happyforms-color-part-placeholder' );

			if ( options && options.placeholder ) {
				placeholderColor = options.placeholder;
			}

			var elementsStyle = {
				base: {
					fontSize: partComputedStyle.getPropertyValue( 'font-size' ).trim(),
					fontFamily: partComputedStyle.getPropertyValue( 'font-family' ),
					color: partComputedStyle.getPropertyValue( 'color' ),
					'::placeholder': {
						color: placeholderColor
					}
				},
				invalid: {
					color: partComputedStyle.getPropertyValue( 'color' )
				}
			};

			var paymentsPartID = this.$paymentsPart.data( 'happyforms-id' );
			var partID = 'happyforms-' + this.formID + '_' + paymentsPartID;

			this.elements['cardNumber'] = elements.create( 'cardNumber', { style: elementsStyle } );
			this.elements['cardNumber'].mount( '#' + partID + '_stripe_card_number' );
			this.elements['cardNumber'].on( 'change', this.onElementChange.bind( this ) );

			this.elements['cardExpiry'] = elements.create( 'cardExpiry', { style: elementsStyle } );
			this.elements['cardExpiry'].mount( '#' + partID + '_stripe_card_expiry' );
			this.elements['cardExpiry'].on( 'change', this.onElementChange.bind( this ) );

			this.elements['cardCvc'] = elements.create( 'cardCvc', { placeholder: '', style: elementsStyle } );
			this.elements['cardCvc'].mount( '#' + partID + '_stripe_card_cvc' );
			this.elements['cardCvc'].on( 'change', this.onElementChange.bind( this ) );
		},

		HappyFormsStripe.prototype.setError = function( elementType, message ) {
			var $element = '';

			switch ( elementType ) {
				case 'cardNumber':
					$element = $( '.stripe-card-number', this.$el );
					break;
				case 'cardExpiry':
					$element = $( '.stripe-card-expiry', this.$el );
					break;
				case 'cardCvc':
					$element = $( '.stripe-card-cvc', this.$el );
					break;
			}

			if ( ! $element ) {
				return;
			}

			$error = $element.next( '.happyforms-message-notices' );
			$( '.credit-card-filled', this.$paymentsPart ).val( 0 );

			$( 'h2', $error ).text( message );
			$error.show();
		},

		HappyFormsStripe.prototype.hideError = function( elementType ) {
			var $element = '';

			switch ( elementType ) {
				case 'cardNumber':
					$element = $( '.stripe-card-number', this.$el );
					break;
				case 'cardExpiry':
					$element = $( '.stripe-card-expiry', this.$el );
					break;
				case 'cardCvc':
					$element = $( '.stripe-card-cvc', this.$el );
					break;
			}

			if ( ! $element ) {
				return;
			}

			$error = $element.next( '.happyforms-message-notices' );

			$( 'h2', $error ).text( '' );
			$error.hide();
		},

		HappyFormsStripe.prototype.onElementChange = function( e ) {
			if ( e.complete ) {
				this.setPaymentMethod();
				this.hideError( e.elementType );
				return;
			}

			if ( ! e.error || e.empty ) {
				return;
			}

			this.setError( e.elementType, e.error.message );
		},

		HappyFormsStripe.prototype.createCheckoutData = function() {
			var data = {
				payment_method: '',
			};

			if ( ! Cookies.get( this.checkoutDataKey ) ) {
				Cookies.set( this.checkoutDataKey, data );
			}
		},

		HappyFormsStripe.prototype.updateCheckoutData = function( key, value ) {
			if ( ! Cookies.get( this.checkoutDataKey ) ) {
				this.createCheckoutData();
			}

			var checkoutData = JSON.parse( Cookies.get( this.checkoutDataKey ) );

			checkoutData[key] = value;

			Cookies.set( this.checkoutDataKey, checkoutData );
		},

		HappyFormsStripe.prototype.getCheckoutData = function() {
			if ( ! Cookies.get( this.checkoutDataKey ) ) {
				this.createCheckoutData();
			}

			var data = JSON.parse( Cookies.get( this.checkoutDataKey ) );

			return data;
		},

		HappyFormsStripe.prototype.clearCheckoutData = function() {
			Cookies.remove( this.checkoutDataKey );
		},

		HappyFormsStripe.prototype.setPaymentMethod = async function() {
			var paymentMethod = await this.createPaymentMethod();
			var $submitButton = $( '.happyforms-button--submit', this.$form );

			$submitButton.attr( 'disabled', 'disabled' );

			if ( ! paymentMethod ) {
				return;
			}

			$( '.credit-card-filled', this.$paymentsPart ).val( 1 );
			$submitButton.removeAttr( 'disabled' );

			this.updateCheckoutData( 'payment_method', paymentMethod.id );
		},

		HappyFormsStripe.prototype.handlePayment = async function( e, response ) {
			if ( ! response.success ) {
				return;
			}

			if ( ! response.data.stripe || ! response.data.stripe.intent ) {
				return;
			}

			var self = this;
			var paymentSuccess = false;
			var checkoutData = this.getCheckoutData();

			if ( ! checkoutData.payment_method ) {
				return;
			}

			await this.stripe.handleCardPayment(
				response.data.stripe.intent.secret,
				{
					payment_method: checkoutData.payment_method
				}
			).then( function( result ) {
				if ( result.paymentIntent && 'succeeded' === result.paymentIntent.status ) {
					paymentSuccess = true;
				}

				$.ajax( {
					url: settings.ajaxurl,
					data: {
						action: 'happyforms_stripe_authorize_payment',
						form_id: self.formID,
						response_id: response.data.stripe.response_id,
						intent_id: response.data.stripe.intent.id,
						nonce: settings.nonce,
						success: paymentSuccess
					}
				} ).done( self.onPaymentComplete.bind( self ) );
			} );
		},

		HappyFormsStripe.prototype.onPaymentComplete = function( response ) {
			if ( ! response.data ) {
				return;
			}

			var self = this;

			setTimeout( function() {
				self.clearCheckoutData();
			}, 500 );

			var $formContainer = $( '#happyforms-' + this.formID );
			var $form = $( 'form', $formContainer );

			$( '.happyforms-message-notice:first', $form ).hide();
			$( response.data ).prependTo( $( '.happyforms-flex', $form ) );

			$formContainer.happyFormsStripe();
		},

		HappyFormsStripe.prototype.createPaymentMethod = async function() {
			if ( ! this.elements.cardNumber ) {
				return;
			}

			var paymentMethod = '';

			await this.stripe.createPaymentMethod( 'card', this.elements.cardNumber ).then( function( result ) {
				paymentMethod = result.paymentMethod;
			} );

			return paymentMethod;
		}

		$.fn.happyFormsStripe = function() {
			this.each( function() {
				$.data( this, 'HappyFormsStripe', new HappyFormsStripe( this ) );
			} );
		}

		$( document ).on( 'happyforms-init', '.happyforms-form', function( e ) {
			$( this ).happyFormsStripe();
		} );
	} );
} )( jQuery, Cookies, _happyFormsStripeSettings );
